<?php
/**
 * e107 Helper class
 * <b>CVS Details:</b>
 * <ul>
 * <li>$Source: e:\_repository\e107_plugins/e107helpers/e107Helper_class.php,v $</li>
 * <li>$Date: 2006/08/29 22:56:37 $</li>
 * </ul>
 * @author     $Author: Neil $
 * @version    $Revision: 1.9 $
 * @copyright  Released under the terms and conditions of the GNU General Public License (http://gnu.org).
 * @package e107Helper
 */

/**
 * A Helper class for the e107 CMS system.
 * Aimed at providing seemless integration for plugins to e107 v0.617 and v0.7.
 * Where there are differences, the standard function names, data formats, etc. from v0.7 are used.
 * @package e107Helper
 */
class e107Helper {
   /**#@+
    * @access private
    */
   var $e107v7;         // Boolean flag indicating e107 v0.7 (true) or not (false)

   var $textparser;     // A Text parser object
   var $userclasslist;  // A comma seperated list of calsses available to the current user
   /**#@-*/

   /**
    * Constructor
    */
   function e107Helper() {
      // OK, this is not ideal but I need to read this file without including it so it doesn't screw
      // up version reporting in e107 admin area (when file is required once by admin_siteinfo.sc).
      $content = file_get_contents(e_ADMIN."ver.php");
      $start = strpos($content, "e107info")-1;
      $content = substr($content, $start, strpos($content, ";", $start)-$start+1);
      eval($content);

      // Use a regular expression here?
      $this->e107v7 = (isset($e107info["e107_version"]) && strpos($e107info["e107_version"], "0.7") === false) ? false : true;
   }

   // ************************************************
   // Miscellaneous Helper Functions
   // ************************************************

   /**
    * @return true if current version of e107 is v0.7, otherwise false
    */
   function isV07() {
      return $this->e107v7;
   }

   /**
    * Get a comma seperated list of classes available to the current user
    * @return a comma seperated string of classes available to the current user
    */
   function getUserClassList() {
      if (!isset($this->userclasslist)) {
         // Initialize value first time in
         if ($this->isV07()) {
            $this->userclasslist = USERCLASS_LIST;
         } else {
            // For 0.617 we have to append standard classes the user has to the list of user classes they have
            $this->userclasslist = strtr(USERCLASS, ".", ",");
            $this->userclasslist .= check_class(e_UC_MEMBER  ) ? ",".e_UC_MEMBER   : "";
            $this->userclasslist .= check_class(e_UC_GUEST   ) ? ",".e_UC_GUEST    : "";
            $this->userclasslist .= check_class(e_UC_PUBLIC  ) ? ",".e_UC_PUBLIC   : "";
            $this->userclasslist .= check_class(e_UC_NOBODY  ) ? ",".e_UC_NOBODY   : "";
            $this->userclasslist .= check_class(e_UC_ADMIN   ) ? ",".e_UC_ADMIN    : "";
            $this->userclasslist .= check_class(e_UC_READONLY) ? ",".e_UC_READONLY : "";
         }
      }
      return $this->userclasslist;
   }

   /**
    * Get a list of all directories in a directory/folder
    * @param   string   name of directory/folder to get directory list For
    * @return  array    list of directories
    */
   function getDirList($dir) {
      $handle = opendir($dir);
      while ($file = readdir($handle)) {
         if (is_dir($dir.$file) && $file !='CVS' && $file != "." && $file != ".." && $file != "/" ) {
            $dirs[] = $file;
         }
      }
      closedir($handle);
      sort($dirs);
      return $dirs;
   }

   /**
    * Get a list of all files in a directory/folder
    * @param   string   name of directory/folder to get file list For
    * @return  array    list of filenames
    */
   function getFileList($dir) {
      $handle = opendir($dir);
      while ($file = readdir($handle)) {
         if (is_file($dir.$file) && (eregi(".jpg",$file) || eregi(".gif",$file) || eregi(".png",$file))) {
            $files[] = $file;
         }
      }
      closedir($handle);
      sort($files);
      return $files;
   }

   // ************************************************
   // Text Parser Class Helper Functions
   // ************************************************

   /** @access private */
   function _initTextParser() {
      if (!isset($this->textparser)) {
         // Initialize value first time in
         if ($this->isV07()) {
            $this->textparser = new e_parse();
         } else {
            $this->textparser = new textparse();
         }
      }
   }

   /**
    * Convert to something suitable for storing in the database
    * @param $txt the text to convert
    * @return the converted text as a string
    */
   function tp_toDB($txt) {
      $this->_initTextParser();
      if ($this->isV07()) {
         return $this->textparser->toDB($txt);
      } else {
         return $this->textparser->formtpa($txt);
      }
   }

   /**
    * Convert to something suitable for displaying on a page as static text
    * @param string  the text to convert
    * @param boolean true if text to parse contains BB codes
    * @param integer truncate length, 0 means no truncation
    * @param string  text to be appended if original text is trncated to $len
    * @return the converted text as a string
    */
   function tp_toHTML($txt, $parseBB=false, $len=0, $more="[more]") {
      $this->_initTextParser();
      if ($this->isV07()) {
          $ret = $this->textparser->post_toHTML($txt, $parseBB);
      } else {
         $ret = $this->textparser->tpa($txt);
      }
      if ($len > 0) {
         $ret = $this->textparser->html_truncate($ret, $len, $more);
      }
      return $ret;
   }

   /**
    * Convert to something suitable for displaying in an HTML form (i.e. editable text)
    * @param $txt the text to convert
    * @return the converted text as a string
    */
   function tp_toForm($txt) {
      $this->_initTextParser();
      if ($this->isV07()) {
         return $this->textparser->toForm($txt);
      } else {
         return $this->textparser->formtparev($txt);
      }
   }

   /**
    * Convert to plain text
    * @param $txt the text to convert
    * @return the converted text as a string
    */
   function tp_toText($txt) {
      $this->_initTextParser();
      if ($this->isV07()) {
         return $this->textparser->toText($txt);
      } else {
         return $this->textparser->formtparev($txt);
      }
   }

   // ************************************************
   // Comment Helper Functions
   // ************************************************

   /**
    * Add comments to a plugins
    * <p>This method returns the HTML for a comment form. In addition, it will post comments to the e107v7
    * comments database and get any existing comments for the current item.</p>
    * @param   string   a unique ID for this plugin, maximum of 10 character
    * @param   int      id of the item comments are allowed for
    * @return  string   HTML for existing comments for an item and the comments form to allow new comments to be posted
    */
   function getComment($pluginid, $id) {
      global $pref, $e107cache, $tp;

      // Include the comment class. Normally, this file is included at a global level, so we need to make the variable
      // it decalares global so it is available inside the comment class
      require_once(e_HANDLER."comment_class.php");
      $GLOBALS["comment_shortcodes"] = $comment_shortcodes;

      $pid = 0; // What is this w.r.t. comment table? Parent ID?

      // Define a comment object
      $cobj = new comment();

      // See if we need to post a comment to the database
      if (isset($_POST['commentsubmit'])) {
         $cobj->enter_comment($_POST['author_name'], $_POST['comment'], $pluginid, $id, $pid, $_POST['subject']);
         if ($pref['cachestatus']){
            $e107cache->clear("comment.$pluginid.{$sub_action}");
         }
      }

      // Specific e107 0.617 processing to render existing comments
      if (!$this->isV07()) {
         $query = $pref['nested_comments'] ?
            "comment_item_id='$id' AND comment_type='$pluginid' AND comment_pid='0' ORDER BY comment_datestamp" :
            "comment_item_id='$id' AND comment_type='$pluginid' ORDER BY comment_datestamp";
         unset($text);
         $mysql = new db();
         if ($comment_total = $mysql->db_Select("comments", "*", $query)) {
            $width = 0;
            while ($row = $mysql->db_Fetch()) {
               // ** Need to sort out how to do nested comments here
               if ($pref['nested_comments']) {
                  $text .= $cobj->render_comment($row, $pluginid, "comment", $id, $width, $subject, true);
               } else {
                  $text .= $cobj->render_comment($row, $pluginid, "comment", $id, $width, $subject, true);
               }
            }
            if (ADMIN && getperms("B")) {
               $text .= "<div style='text-align:right'><a href='".e_ADMIN."modcomment.php?$pluginid.$id'>".LAN_314."</a></div>";
            }
         }
      }

      // Get comment form - e107 sends this to the output buffer so we must grab it and assign to our return string
      ob_start();
      if ($this->isV07()) {
         // e107 0.7
         $cobj->compose_comment($pluginid, "comment", $id, $width, $subject, $showrate=false);
      } else {
         // e107 0.617
         if (strlen($text) > 0) {
            $ns = new e107table();
            $ns->tablerender(LAN_5, $text);
         }
         $cobj->form_comment("comment", $pluginid, $id, $subject, $content_type);
      }
      $text = ob_get_contents();
      ob_end_clean();

      return $text;
   }

   /**
    * Add ratings to a plugins
    * <p>This method returns the HTML for a ratings form. In addition, it will post ratings to the e107v7
    * ratings database and get any existing ratings for the current item.</p>
    * @param   string   a unique ID for this plugin, maximum of 10 character
    * @param   int      id of the item comments are allowed for
    * @param   boolean  true to show the rating selection drop down if user not already rated this item
    * @return  string   HTML for existing comments for an item and the comments form to allow new comments to be posted
    */
   function getRating($pluginid, $id, $allowRating=true) {
      $rater = new rater();

      $text = "";
      //$text .= "<table><tr><td colspan='2' style='text-align:right'>";
      if ($ratearray = $rater->getrating($pluginid, $id)) {
         for($c = 1; $c <= $ratearray[1]; $c++) {
            $text .= "<img src='".e_IMAGE."rate/".IMODE."/star.png' alt='' />";
         }
         if ($ratearray[2]) {
            $text .= "<img src='".e_IMAGE."rate/".IMODE."/".$ratearray[2].".png'  alt='' />";
         }
         if ($ratearray[2] == "") {
            $ratearray[2] = 0;
         }
         $text .= "&nbsp;".$ratearray[1].".".$ratearray[2]." - ".$ratearray[0]."&nbsp;";
         $text .= ($ratearray[0] == 1 ? HELPER_RATELAN_0 : HELPER_RATELAN_1);
      } else {
         $text .= HELPER_RATELAN_4;
      }

      if ($allowRating) {
         if (!$rater->checkrated($pluginid, $id) && USER) {
            $ratetext = $rater->rateselect("&nbsp;&nbsp;&nbsp;&nbsp;<b>".HELPER_RATELAN_2, $pluginid, $id)."</b>";
            if (!$this->isV07()) {
               $ratetext = str_replace("rate.php", "../../rate.php", $ratetext);
            }
            $text .= $ratetext;
         } else if (!USER) {
            $text .= "&nbsp;";
         } else {
            $text .= "&nbsp;-&nbsp;".HELPER_RATELAN_3;
         }
      }

      return $text;
   }

   /**
    * Add an image that toggles the actual picture on each mouse click
    * <p>This method returns the HTML for a toggle image.</p>
    * @param   string   a unique ID for this plugin, maximum of 10 character
    * @param   int      id of the item comments are allowed for
    * @param   boolean  true to show the rating selection drop down if user not already rated this item
    * @return  string   HTML for existing comments for an item and the comments form to allow new comments to be posted
    */
   function getToggleImage($id, $img1, $img2) {
      return "<div id='$id'><img src='$img1' border='0' onclick='e107HelperAjax.toggleImage(\"".e_PLUGIN."e107helpers/e107HelperAjax.php\", \"$id\", \"$img1\", \"$img2\")'></div>";
   }

   /**
    * Generate HTML for a textarea
    * <p>Allows control of various display options including BB helper toolbar and emote icon helper toobar.</p>
    * @param   text        the text to be displayed in the textarea
    * @param   name        name attribute for the textarea (defaults to e107heleprTA)
    * @param   class       style class attribute value (defaults to tbox)
    * @param   rows        number of row to be displayed (defaults to 15)
    * @param   cols        number of columns to be displayed (defaults to false, meaning no cols attribute required)
    * @param   width       width of the textarea (e.g. 98%, 400px) (defaults to false, meaning no width required)
    * @param   helpMode    display textarea help toolbar 1=yes, 2=yes with tooltips (defaults to false, meaning no toolbar)
    * @param   showEmotes  display BB/emote icon helper toobar (true) or not (false, the default)
    * @return  string      the HTML for the textarea
    */
   function getTextarea($tatext="", $name="e107heleprTA", $class="tbox", $rows="15", $cols=false, $width=false, $helpMode=false, $showEmotes=false) {
      $text = "";
      $text .= "<textarea name='$name' class='$class' rows='$rows' onselect='storeCaret(this);' onclick='storeCaret(this);' onkeyup='storeCaret(this);'";
      if ($width) {
         $text .= " style='width:$width;'";
      }
      if ($cols) {
         $text .= " cols='$cols'";
      }
      $text .= ">$tatext</textarea>";

      if ($helpMode) {
         if ($this->isV07()) {
            $text .= display_help($name, $helpMode);
         } else {
            $text .= ren_help($helpMode);
         }
      }
      if ($showEmotes) {
         $text .= r_emote();
      }
      return $text;
   }

   /**
    * Send a notification to one or more users.
    * <p>Current implementation just sends a Private Message, so the PM plugin must be enabled.</p>
    * @param   userids     the userid(s) to send notifications to
    * @param   subject     the subject of the message
    * @param   message     the message itself
    * @param   userclass
    * @return  TBC
    * @TODO add option to PM multiple users
    * @TODO add option to PM a userclass
    * @TODO proper return values
    * @TODO localization
    */
   function sendNotification($userids, $subject, $message, $userclass) {
      global $pm_prefs, $pm, $pref, $tp;

      // Caller must pass in a user class to ensure PMs are allowed to be sent by this user
      if (!check_class($userclass)) {
         return NOT_AUTHORIZED;
      }

      $mysql = new e107helperDB();
		if (!$mysql->db_Select("user", "user_id, user_name, user_class, user_email", "user_id=$userids")) {
		   return USER_NOT_FOUND;
		}
		$touser = $mysql->db_Fetch();

      // Annotate message with senders details
      if (USERID !== false) {
         $message = "Notification from ".USERNAME."\n\n".$message;
      } else {
         $message = "Notification from Guest\n\n".$message;
      }

      $pm = new private_message();
      $vars = array(
         "pm_subject"   => $subject,
         "pm_message"   => $message,
         "from_id"      => 0,                   // ensure sender does not get a copy of the notification in their outbox
         "to_info"      => array(
            "user_id"   => $touser["user_id"],
            "user_name" => $touser["user_name"]
         )
      );
      return $pm->add($vars);
   }

   // experimental - subject to change
   function getTooltip($text, $caption="") {
      $text = "domTT_activate(this,event";
      $text .= ",\"content\",\"$text\"";
      if (strlen($caption) > 0) {
         $text .= ",\"caption\",\"$caption\"";
      }
      $text .= ")";
      return $text;
   }
}
?>